package diffpatch;

import org.w3c.dom.events.*;
import org.w3c.dom.events.MutationEvent.*;
import org.w3c.dom.*;
import org.w3c.dom.Node.*;

import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import java.io.*;
import java.util.*;
import javax.swing.SwingUtilities;

// Application related packages. 
import editor.*;

/* The basic idea is as follows:
   1) The event listener handles the events fired when XMLPatch 
      processes the synchronization message.
   2) This is application specific; this demonstrates how it 
      can be done.
*/      
public class PatchListener implements org.w3c.dom.events.EventListener {
   ShapeMgr shMgr; 
   MutationEvent mevt; 
   String targetName; 
   /* To ensure that the shape  is drawn only, after getting all the coordnates;
      here its 4.
   */
   static int attrEvtCount = 0;

   /* When attributes that have been changed alone are transmitted,
    * the number of attributes transmitted must be known
    */
   int attrsChangedCount = 0;
    
   public PatchListener(ShapeMgr shapemgr ) {
    shMgr = shapemgr; 
   }
   
   public void setAffectedNodeName(String name) {
     targetName = name;
   }

   public void setAttributesCount(int count) {
     attrsChangedCount = count;
   }

   public void handleEvent(org.w3c.dom.events.Event evt) {
      Element target;
      String shape; 

      mevt = (MutationEvent)evt; 
      target = (Element)(mevt.getTarget());
      shape = target.getNodeName();
     
      /* The actions that are to be trapped are  <attr-modify>/<attr-add>, as the shapes 
         are defined by these attributes; 
      */  
       
      if ( mevt.getType().equals("DOMAttrModified") )  {
	 int x,y, w, h;
	     switch ( mevt.getAttrChange())   {
                case MutationEvent.ADDITION: {
                  /*  call the manage() of ShapeMgr here, with a flag
		      such that it doesn't record it in app-DOM: 
		      i-e:-  to differentiate between the actions done by the app. 
		      AND the one done by the remote client.
		  */ 
                  attrEvtCount++;
                  if (attrEvtCount == 4) {
                    int coords[] = new int[4];
		    System.arraycopy(getCoords(target), 0, coords, 0, 4);
	   	    if ( shape.equals("rectangle"))   
                      shMgr.manage(new RectangleShape(coords[0], coords[1], 
				  coords[2], coords[3]), true); 
		    else if ( shape.equals("ellipse")) 
	              shMgr.manage(new EllipseShape(coords[0], coords[1], 
			          coords[2], coords[3]), true);
		    else
		      shMgr.manage(new LineShape(coords[0], coords[1], 
				  coords[2], coords[3]), true);
  
  	            attrEvtCount = 0;
		  }  
		  break;
                 }
					     
                 case MutationEvent.MODIFICATION:  {
                  attrEvtCount++;
                  if (attrEvtCount == attrsChangedCount) {	
                    int coords[] = new int[4];
                    System.arraycopy(getCoords(target), 0, coords, 0, 4);						   
                    shMgr.simulateRepaint(targetName, coords[0], coords[1],
				        coords[2], coords[3]);
                    attrEvtCount = 0; 
                  }
		  break;					    
	         }
	    } //switch	
      // }// if attrEvtCount
     } // if mevt.getType()					     
   } //handleEvent


   /* Gets the coordinates of the shape given the shape name */
   public int[] getCoords(Element target) { 
     int []coords = new int[4]; 
     String shape = target.getNodeName();
     
     if (shape.equals("rectangle")) {
        coords[0] = Integer.parseInt( target.getAttribute("x"));
	coords[1] = Integer.parseInt( target.getAttribute("y"));
	coords[2] = Integer.parseInt( target.getAttribute("width"));
	coords[3] = Integer.parseInt( target.getAttribute("height"));
     }
     else if (shape.equals("ellipse")) {
       coords[0] = Integer.parseInt( target.getAttribute("x"));
       coords[1] = Integer.parseInt( target.getAttribute("y"));
       coords[2] = Integer.parseInt( target.getAttribute("w"));
       coords[3] = Integer.parseInt( target.getAttribute("h"));

     }
     else if (shape.equals("line")) {
       coords[0] = Integer.parseInt( target.getAttribute("x1"));
       coords[1] = Integer.parseInt( target.getAttribute("y1"));
       coords[2] = Integer.parseInt( target.getAttribute("x2"));
       coords[3] = Integer.parseInt( target.getAttribute("y2"));
				  
     }
    return (coords);
   } //getCoords()


} //PatchListenaer
