package editor;

/* All the classes that belong to this application fall under this directory. */
import java.awt.*;
import java.util.*;
import java.awt.event.*;
import sharedboard.*; 

 /* The basic idea is as follows:
    1) The application is a stand-alone drawing program, so I have made many 
       changes to make it collaborative, and gluing it with the code in 
       /diffpatch. 
    2) Every time an object is created, a reference to it is stored by hashing 
       its name (say, line[0]) to the object. On synchronization, the object is looked-up
       and is suitably re-drawn.
    Please refer to the link 
    "http://www.javaworld.com/javaworld/jw-12-1996/jw-12-howto.html" 
    for addtional details; only brief comments toc hnages made (highlighted by,
    GK) are provided.
*/ 

public class ShapeMgr extends Canvas implements 
             MouseListener, MouseMotionListener   
{
   Vector _vec;
   HotSpotMgr _hsm;
   HotSpot _hs;
   
   /* GK */ 
   Shape   currShape;
   Shape currRectShape, currEllipseShape, currLineShape;
   boolean colorChosen = false; 
   Vector shapeNames;
   WBDomManager wbDOMMgr; 
   String currShapeName = "asdf"; 
   Color chooseColor;
	   
   /* Useful in inferring that the object was dragged
      and repositioned */
   boolean isMouseDragged = false; 
   Hashtable shapeTable; 
   /* Each shape added (rectangle[0], ellipse[1], 
      has its object's reference as the hash-value
   */
   int shapeCnt = 0;
   
   public ShapeMgr()
   {
      _vec = new Vector();
      _hsm = new HotSpotMgr();

     //GK
     shapeNames = new Vector();
     addMouseListener(this);
     addMouseMotionListener(this);
     shapeTable = new Hashtable();
   }

   //GK
   /* isRemoteCall -    
      Call the manage method here, with a flag
      such that it doesn't record it in DOM -
      i-e:-  to differentiate between the actions
      done by the app. AND the one done by the remote client. 
   */

   public void manage(Shape s, boolean isRemoteCall)
   {
      _vec.addElement(s);
      
      Vector vecShapes = s.getHotSpots();
      for (int i = 0; i < vecShapes.size(); i++)
      {
         _hsm.manage((HotSpot)vecShapes.elementAt(i));
      }
      repaint(); 
     
      //GK
      AttribValue attribval = null;
      String hashKey = null; 

      /* shapeCnt keeps track of the newly created shape's pos in the list
         => as and when objs are created. 
      */ 
      if (s.getClass().getName().endsWith("RectangleShape")) {
         currShapeName = "rectangle";
	 currRectShape = s;
      }
      else if (s.getClass().getName().endsWith("LineShape")){ 
	 currShapeName = "line";
	 currLineShape = s;
      }
      else if (s.getClass().getName().endsWith("EllipseShape")){
         currShapeName = "ellipse";	   
	 currEllipseShape = s;
      }
      else {
         ;
      }

      // GK -hashing the recently formed obj. with name as rectangle[0], etc
      hashKey = getHashKey(currShapeName, shapeCnt);
      shapeTable.put(hashKey, s);
      if (currShapeName.equals("rectangle"))
          ((RectangleShape)s).setOrderedName(hashKey);
      else if (currShapeName.equals("ellipse")) 
          ((EllipseShape)s).setOrderedName(hashKey);
      else if (currShapeName.equals("line"))
          ((LineShape)s).setOrderedName(hashKey);
      
      shapeCnt++;
      if ( !isRemoteCall ) {
         if ( currShapeName.equals("rectangle") )
	   // Include the color attribute here.
           attribval = (AttribValue)getAttribValue("x", ((RectangleShape)currRectShape)._x,
			              "y", ((RectangleShape)currRectShape)._y,
				      "width", ((RectangleShape)currRectShape)._w,
				      "height", ((RectangleShape)currRectShape)._h
			             );
	 else if (  currShapeName.equals("line") )
	   attribval = 	(AttribValue)getAttribValue("x1", ((LineShape)currLineShape)._x1,
			               "y1", ((LineShape)currLineShape)._y1,
				       "x2", ((LineShape)currLineShape)._x2,
				       "y2", ((LineShape)currLineShape)._y2
				      );
	 else if ( currShapeName.equals("ellipse") )
           attribval = (AttribValue)getAttribValue("x", ((EllipseShape)currEllipseShape)._x,
				      "y", ((EllipseShape)currEllipseShape)._y,
			              "w", ((EllipseShape)currEllipseShape)._w,
				      "h", ((EllipseShape)currEllipseShape)._h
			             );
			               
        wbDOMMgr.addChild(currShapeName); 
	wbDOMMgr.updateAttribute(hashKey,attribval.getVector(),1);
	wbDOMMgr.sendToServer();
      }
   } //manage


   //GK
   // Returns the name formtted with the position within the list.
   public String getHashKey(String name, int cnt) {
     return(name + "[" + String.valueOf(cnt) + "]");
	   
   }
   
   //GK
   public Object getAttribValue(String coord1, int x, 
		                String coord2, int y,
		                String coord3, int param3, 
				String coord4, int param4) {
  
	   return ( new AttribValue( coord1, x, coord2, y, coord3, param3, coord4, param4 )
	          );
   }
   
   //GK
   // Sets up socket connection with the peer.
   public void register( String server,
                        int appServPort) {
      wbDOMMgr = new WBDomManager(this,  server, appServPort);
   }

   //GK
   // Sends the sync. mesage to peer.
   public void sendWBMsgToServer() {
    wbDOMMgr.sendToServer(); 
   } 

   public void mouseClicked(MouseEvent me) {
    ;
   } 

   public void mousePressed(MouseEvent me) {
     _hs = _hsm.isAt(new Coordinates(me.getX(), me.getY()));
   } 

   public void mouseExited(MouseEvent me) {
      ; 
   } 

   public void mouseEntered(MouseEvent me) {
      ;
   }
   
   public void mouseReleased(MouseEvent me) {
     //GK
     AttribValue attribVal = null;
     _hs = null;
     Shape shapeRef = null; 
    
     //GK 
     // Look up the hashtable to get the object hashed earlier.
     shapeRef = (Shape)(shapeTable.get(HotSpot.currShapeName)); 
     if (isMouseDragged) { 
        if (HotSpot.currShapeName.startsWith("rectangle")) {
           attribVal = new AttribValue("x", ((RectangleShape)shapeRef)._x, 
                                       "y", ((RectangleShape)shapeRef)._y,
                                       "width", ((RectangleShape)shapeRef)._w, 
                                       "height", ((RectangleShape)shapeRef)._h
                                    );
        } 
        else if (HotSpot.currShapeName.startsWith("line") ) {
           attribVal = new AttribValue("x1", ((LineShape)shapeRef)._x1,
                                       "y1", ((LineShape)shapeRef)._y1,
                                       "x2", ((LineShape)shapeRef)._x2,
                                       "y2", ((LineShape)shapeRef)._y2
                                      );
        }
        else if (HotSpot.currShapeName.startsWith("ellipse")) {
          attribVal = new AttribValue("x", ((EllipseShape)shapeRef)._x,
                                      "w", ((EllipseShape)shapeRef)._w,
                                      "y", ((EllipseShape)shapeRef)._y,
                                      "h", ((EllipseShape)shapeRef)._h
                                     );
        }
        wbDOMMgr.updateAttribute(HotSpot.currShapeName,attribVal.getVector(), 2);
	wbDOMMgr.sendToServer();
     } //if 
   }
 
   public void mouseDragged(MouseEvent me) {
      if (_hs != null)
      {
         _hs.setCoordinates(new Coordinates(me.getX(), me.getY()));
         _hs.notifyObservers();
         repaint();
        
	 //GK
	 // Tells that the mouse was dragged.
         isMouseDragged = true; 
 
      }
   } 

   public void mouseMoved(MouseEvent me) {
     ;
   } 

   //GK 
   // Called by the PatchListener to draw the shape in the peer, for synchronization.
   public void simulateRepaint(String shapeName, int xcoord, int ycoord,
		               int wcoord, int hcoord) {
     /* Get the latest x,y,w,h coords as it would have at the sending side */ 
     currShapeName = shapeName.substring(0, shapeName.indexOf('['));
     String decodedShapeName =  shapeName.substring(0, shapeName.indexOf('['));
     Shape shapeRef, rectShapeRef, ellipseShapeRef, lineShapeRef; 
	    
     shapeRef = (Shape)shapeTable.get(shapeName);
     //System.out.println("The decodedSHape::shapeName:: " + decodedShapeName + ";;" + shapeName);
     if (decodedShapeName.equals("rectangle") ) {
        ((RectangleShape)shapeRef).setCoords(xcoord,ycoord,wcoord,hcoord);	
     }
     else if (decodedShapeName.equals("ellipse")) {
       ((EllipseShape)shapeRef).setCoords(xcoord,ycoord,wcoord,hcoord);
     }
     else {
       ((LineShape)shapeRef).setCoords(xcoord,ycoord,wcoord,hcoord);
     }
     repaint();
   }
  
   /* public void simulateMouseUp() {
       _hs = null;
   }
 
   public void simulateMouseDown( int x, int y) {
      _hs = _hsm.isAt(new Coordinates(x, y));
   }

    public void  simulateMouseDrag(int x, int y) {
      System.out.println(" _hs = " + _hs);
         _hs.setCoordinates(new Coordinates(x, y));
         _hs.notifyObservers();
         System.out.println("after notify observers..from silmuMouseDrag"); 
         repaint();
	 System.out.println("hs != null");
    }
   */
   
   public void paint(Graphics g)
   {
      for (int i = 0; i < _vec.size(); i++)
      {
         Shape s = (Shape)_vec.elementAt(i);

         g.setColor(Color.lightGray);

         Vector vecShapes = s.getHotSpots();

         for (int j = 0; j < vecShapes.size(); j++)
         {
            HotSpot hs = (HotSpot)vecShapes.elementAt(j);

            hs.paint(g);
         }

         g.setColor(Color.blue);
	 s.paint(g);
      }
   }
   
} //ShapeMgr.java
