package editor;

import org.w3c.dom.events.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import java.io.*;
import java.util.*;

import diffpatch.XMLDiff; 
import diffpatch.XMLPatch;
import sharedboard.WBAppServer;


public class WBDomManager {
Document doc;
Node domRoot,  child1, child2, child3, child4, child5, child6;
XMLDiff xmldiff;
XMLPatch saxproc;

/** Class WBDomManager acts like an interface between the application 
 *  and the diff-patch module; the following shows how it is done 
 *  for the given application, EditGUI.
 */  
public WBDomManager(ShapeMgr shapeMgr, String server, 
                    int appServPort ) { 
 try {
  DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
  DocumentBuilder db = dbf.newDocumentBuilder();
  doc = db.newDocument();
  domRoot = doc.createElement("svg");
  doc.appendChild(domRoot);
 } 
 catch (Exception excp) {
    excp.printStackTrace();
 } 

 
 xmldiff = new XMLDiff(doc, server, 
                       appServPort, shapeMgr);
 /* Instantiate the saxproc inside the server */
 Thread wbAppServer = new WBAppServer(appServPort, doc, 
                                      shapeMgr, xmldiff);
 wbAppServer.start();

}

public void sendToServer() { 
 xmldiff.sendToWBServer();
}

/* The actions that most likely to be called are
   appendChild(), removeChild(), updateAttribs();
   with the corresponding arguments...
   Assumptions: All nodes are added to the <svg> root element by default:
   Advanced apps. can manage it easily. 
*/

 public void addChild(String nodeName) {
   Node child = doc.createElement(nodeName);
   domRoot.appendChild(child);
 }

 public void deleteChild(String nodeName) {
  Node node = getNodeReference(nodeName);
  domRoot.removeChild(node);
 }

 public void updateAttribute(String nodeName, Vector attribval, int type) {
 //1 - set/add;2- modify; 3 - remove
 Node node = getNodeReference(nodeName);
 for(int i=0;i < attribval.capacity(); i++) {
 Compound pair = (Compound)attribval.elementAt(i);
  if (type == 1 || type == 2 )  {
   ((Element)node).setAttribute(pair.attribname, pair.attribval);
  }
  else {
   ((Element)node).removeAttribute(pair.attribname);
  }
 } //for
} 
   
public Node getNodeReference(String nodeName) {
  Node findNode = null;
  NodeList list = domRoot.getChildNodes();
  String findNodeName = nodeName.substring(0, nodeName.indexOf('['));
  int nodePos  = Integer.parseInt(nodeName.substring(nodeName.indexOf('[') + 1, 
			          nodeName.lastIndexOf(']')));
  findNode = list.item(nodePos);
  return (findNode);
}

} //End-of-class

